@@ -1,6 +1,6 @@
 # encoding: utf-8
 """
-capability.py
+capability/__init__.py
 
 Created by Thomas Mangin on 2012-07-17.
 Copyright (c) 2009-2013 Exa Networks. All rights reserved.
@@ -8,21 +8,23 @@ Copyright (c) 2009-2013 Exa Networks. All rights reserved.
 
 from struct import unpack
 
-from exabgp.structure.utils import hexa
-
 from exabgp.protocol.family import AFI,SAFI
 from exabgp.bgp.message.open.asn import ASN
 from exabgp.bgp.message.open.capability.id import CapabilityID
 from exabgp.bgp.message.open.capability.mp import MultiProtocol
-from exabgp.bgp.message.open.capability.refresh import RouteRefresh
+from exabgp.bgp.message.open.capability.refresh import RouteRefresh,EnhancedRouteRefresh
 from exabgp.bgp.message.open.capability.graceful import Graceful
 from exabgp.bgp.message.open.capability.ms import MultiSession
 from exabgp.bgp.message.open.capability.addpath import AddPath
+from exabgp.bgp.message.open.capability.operational import Operational
 from exabgp.bgp.message.notification import Notify
 
+def hexa (value):
+	return "%s" % [(hex(ord(_))) for _ in value]
+
 # =================================================================== Unknown
 
-class Unknown (object):
+class UnknownCapability (object):
 	def __init__ (self,value,raw=''):
 		self.value = value
 		self.raw = raw
@@ -49,6 +51,14 @@ class Parameter (int):
 # =================================================================== Capabilities
 # http://www.iana.org/assignments/capability-codes/
 
+# +------------------------------+
+# | Capability Code (1 octet)    |
+# +------------------------------+
+# | Capability Length (1 octet)  |
+# +------------------------------+
+# | Capability Value (variable)  |
+# +------------------------------+
+
 class Capabilities (dict):
 	def announced (self,capability):
 		return capability in self
@@ -63,6 +73,8 @@ class Capabilities (dict):
 				r += [str(self[key])]
 			elif key == CapabilityID.CISCO_ROUTE_REFRESH:
 				r += ['Cisco Route Refresh']
+			elif key == CapabilityID.ENHANCED_ROUTE_REFRESH:
+				r += ['Enhanced Route Refresh']
 			elif key == CapabilityID.GRACEFUL_RESTART:
 				r += ['Graceful Restart']
 			elif key == CapabilityID.FOUR_BYTES_ASN:
@@ -73,6 +85,8 @@ class Capabilities (dict):
 				r += ['Multi Session']
 			elif key == CapabilityID.ADD_PATH:
 				r += [str(self[key])]
+			elif key == CapabilityID.OPERATIONAL:
+				r += ['Operational']
 			elif key in CapabilityID.reserved:
 				r += ['private use capability %d' % key]
 			elif key in CapabilityID.unassigned:
@@ -96,6 +110,8 @@ class Capabilities (dict):
 			ap_families = []
 			if (AFI(AFI.ipv4),SAFI(SAFI.unicast)) in families:
 				ap_families.append((AFI(AFI.ipv4),SAFI(SAFI.unicast)))
+			if (AFI(AFI.ipv6),SAFI(SAFI.unicast)) in families:
+				ap_families.append((AFI(AFI.ipv6),SAFI(SAFI.unicast)))
 			if (AFI(AFI.ipv4),SAFI(SAFI.nlri_mpls)) in families:
 				ap_families.append((AFI(AFI.ipv4),SAFI(SAFI.nlri_mpls)))
 			#if (AFI(AFI.ipv6),SAFI(SAFI.unicast)) in families:
@@ -109,7 +125,8 @@ class Capabilities (dict):
 				self[CapabilityID.GRACEFUL_RESTART] = Graceful(0x0,graceful,[(afi,safi,Graceful.FORWARDING_STATE) for (afi,safi) in families])
 
 		if neighbor.route_refresh:
-			self[CapabilityID.ROUTE_REFRESH] = RouteRefresh(families)
+			self[CapabilityID.ROUTE_REFRESH] = RouteRefresh()
+			self[CapabilityID.ENHANCED_ROUTE_REFRESH] = EnhancedRouteRefresh()
 
 		# MUST be the last key added
 		if neighbor.multisession:
@@ -124,88 +141,103 @@ class Capabilities (dict):
 		parameters = "".join(["%s%s%s" % (chr(2),chr(len(r)),r) for r in rs])
 		return "%s%s" % (chr(len(parameters)),parameters)
 
-	def _key_values (self,name,data):
-		if len(data) < 2:
-			raise Notify(2,0,"Bad length for OPEN %s (<2) %s" % (name,hexa(data)))
-		l = ord(data[1])
-		boundary = l+2
-		if len(data) < boundary:
-			raise Notify(2,0,"Bad length for OPEN %s (buffer underrun) %s" % (name,hexa(data)))
-		key = ord(data[0])
-		value = data[2:boundary]
-		rest = data[boundary:]
-		return key,value,rest
 
-	def factory (self,data):
-		option_len = ord(data[0])
-		if option_len:
-			data = data[1:]
-			while data:
-				key,value,data = self._key_values('parameter',data)
-				# Paramaters must only be sent once.
-				if key == Parameter.AUTHENTIFICATION_INFORMATION:
-					raise Notify(2,5)
 
-				if key == Parameter.CAPABILITIES:
-					while value:
-						k,capv,value = self._key_values('capability',value)
-						# Multiple Capabilities can be present in a single attribute
-						#if r:
-						#	raise Notify(2,0,"Bad length for OPEN %s (size mismatch) %s" % ('capability',hexa(value)))
+def _key_values (name,data):
+	if len(data) < 2:
+		raise Notify(2,0,"Bad length for OPEN %s (<2) %s" % (name,hexa(data)))
+	l = ord(data[1])
+	boundary = l+2
+	if len(data) < boundary:
+		raise Notify(2,0,"Bad length for OPEN %s (buffer underrun) %s" % (name,hexa(data)))
+	key = ord(data[0])
+	value = data[2:boundary]
+	rest = data[boundary:]
+	return key,value,rest
 
-						if k == CapabilityID.MULTIPROTOCOL_EXTENSIONS:
-							if k not in self:
-								self[k] = MultiProtocol()
-							afi = AFI(unpack('!H',capv[:2])[0])
-							safi = SAFI(ord(capv[3]))
-							self[k].append((afi,safi))
-							continue
+def CapabilitiesFactory (data):
+	capabilities = Capabilities()
 
-						if k == CapabilityID.GRACEFUL_RESTART:
-							restart = unpack('!H',capv[:2])[0]
-							restart_flag = restart >> 12
-							restart_time = restart & Graceful.TIME_MASK
-							value_gr = capv[2:]
-							families = []
-							while value_gr:
-								afi = AFI(unpack('!H',value_gr[:2])[0])
-								safi = SAFI(ord(value_gr[2]))
-								flag_family = ord(value_gr[0])
-								families.append((afi,safi,flag_family))
-								value_gr = value_gr[4:]
-							self[k] = Graceful(restart_flag,restart_time,families)
-							continue
+	option_len = ord(data[0])
+	if option_len:
+		data = data[1:]
+		while data:
+			key,value,data = _key_values('parameter',data)
+			# Paramaters must only be sent once.
+			if key == Parameter.AUTHENTIFICATION_INFORMATION:
+				raise Notify(2,5)
 
-						if k == CapabilityID.FOUR_BYTES_ASN:
-							self[k] = ASN(unpack('!L',capv[:4])[0])
-							continue
+			if key == Parameter.CAPABILITIES:
+				while value:
+					k,capv,value = _key_values('capability',value)
+					# Multiple Capabilities can be present in a single attribute
+					#if r:
+					#	raise Notify(2,0,"Bad length for OPEN %s (size mismatch) %s" % ('capability',hexa(value)))
 
-						if k == CapabilityID.ROUTE_REFRESH:
-							self[k] = RouteRefresh()
-							continue
+					if k == CapabilityID.MULTIPROTOCOL_EXTENSIONS:
+						if k not in capabilities:
+							capabilities[k] = MultiProtocol()
+						afi = AFI(unpack('!H',capv[:2])[0])
+						safi = SAFI(ord(capv[3]))
+						capabilities[k].append((afi,safi))
+						continue
 
-						if k == CapabilityID.CISCO_ROUTE_REFRESH:
-							self[k] = RouteRefresh()
-							continue
+					if k == CapabilityID.GRACEFUL_RESTART:
+						restart = unpack('!H',capv[:2])[0]
+						restart_flag = restart >> 12
+						restart_time = restart & Graceful.TIME_MASK
+						value_gr = capv[2:]
+						families = []
+						while value_gr:
+							afi = AFI(unpack('!H',value_gr[:2])[0])
+							safi = SAFI(ord(value_gr[2]))
+							flag_family = ord(value_gr[0])
+							families.append((afi,safi,flag_family))
+							value_gr = value_gr[4:]
+						capabilities[k] = Graceful(restart_flag,restart_time,families)
+						continue
 
-						if k == CapabilityID.MULTISESSION_BGP:
-							self[k] = MultiSession()
-							continue
-						if k == CapabilityID.MULTISESSION_BGP_RFC:
-							self[k] = MultiSession()
-							continue
-						if k == CapabilityID.ADD_PATH:
-							self[k] = AddPath()
-							value_ad = capv
-							while value_ad:
-								afi = AFI(unpack('!H',value_ad[:2])[0])
-								safi = SAFI(ord(value_ad[2]))
-								sr = ord(value_ad[3])
-								self[k].add_path(afi,safi,sr)
-								value_ad = value_ad[4:]
+					if k == CapabilityID.FOUR_BYTES_ASN:
+						capabilities[k] = ASN(unpack('!L',capv[:4])[0])
+						continue
 
-						if k not in self:
-							self[k] = Unknown(k,[ord(_) for _ in capv])
-				else:
-					raise Notify(2,0,'Unknow OPEN parameter %s' % hex(key))
-		return self
+					if k == CapabilityID.CISCO_ROUTE_REFRESH:
+						capabilities[k] = RouteRefresh()
+						continue
+
+					if k == CapabilityID.ROUTE_REFRESH:
+						capabilities[k] = RouteRefresh()
+						continue
+
+					if k == CapabilityID.ENHANCED_ROUTE_REFRESH:
+						capabilities[k] = EnhancedRouteRefresh()
+						continue
+
+					if k == CapabilityID.MULTISESSION_BGP:
+						capabilities[k] = MultiSession()
+						continue
+
+					if k == CapabilityID.MULTISESSION_BGP_RFC:
+						capabilities[k] = MultiSession()
+						continue
+
+					if k == CapabilityID.ADD_PATH:
+						if k not in capabilities:
+							capabilities[k] = AddPath()
+						value_ad = capv
+						while value_ad:
+							afi = AFI(unpack('!H',value_ad[:2])[0])
+							safi = SAFI(ord(value_ad[2]))
+							sr = ord(value_ad[3])
+							capabilities[k].add_path(afi,safi,sr)
+							value_ad = value_ad[4:]
+
+					if k == CapabilityID.OPERATIONAL:
+						capabilities[k] = Operational()
+						continue
+
+					if k not in capabilities:
+						capabilities[k] = UnknownCapability(k,[ord(_) for _ in capv])
+			else:
+				raise Notify(2,0,'Unknow OPEN parameter %s' % hex(key))
+	return capabilities
