@@ -20,6 +20,8 @@ class CapabilityID (object):
 	DYNAMIC_CAPABILITY       = 0x43  # [Chen]
 	MULTISESSION_BGP_RFC     = 0x44  # [draft-ietf-idr-bgp-multisession]
 	ADD_PATH                 = 0x45  # [draft-ietf-idr-add-paths]
+	ENHANCED_ROUTE_REFRESH   = 0x46  # [draft-ietf-idr-bgp-enhanced-route-refresh]
+	OPERATIONAL              = 0x47  # ExaBGP only ...
 	# 70-127    Unassigned
 	CISCO_ROUTE_REFRESH      = 0x80  # I Can only find reference to this in the router logs
 	# 128-255   Reserved for Private Use [RFC5492]
@@ -29,3 +31,6 @@ class CapabilityID (object):
 
 	unassigned = range(70,128)
 	reserved = range(128,256)
+
+from exabgp.util.enumeration import Enumeration
+REFRESH = Enumeration ('absent','normal','enhanced')
