@@ -6,7 +6,6 @@ Created by Thomas Mangin on 2009-11-05.
 Copyright (c) 2009-2013 Exa Networks. All rights reserved.
 """
 
-from exabgp.protocol.ip.address import Address
 from exabgp.bgp.message.update.attribute.id import AttributeID
 from exabgp.bgp.message.update.attribute import Flag,Attribute
 
@@ -17,14 +16,25 @@ class MPURNLRI (Attribute):
 	ID = AttributeID.MP_UNREACH_NLRI
 	MULTIPLE = True
 
-	def __init__ (self,routes):
-		self.routes = routes
+	def __init__ (self,nlris):
+		self.nlris = nlris
+
+	def packed_attributes (self,addpath):
+		if not self.nlris:
+			return
+
+		mpurnlri = {}
+		for nlri in self.nlris:
+			mpurnlri.setdefault((nlri.afi.pack(),nlri.safi.pack()),[]).append(nlri.pack(addpath))
+
+		for (pafi,psafi),nlris in mpurnlri.iteritems():
+			yield self._attribute(pafi + psafi + ''.join(nlris))
 
 	def pack (self,addpath):
-		return self._attribute(self.routes[0].nlri.afi.pack() + self.routes[0].nlri.safi.pack() + ''.join([route.nlri.pack(addpath) for route in self.routes]))
+		return ''.join(self.packed_attributes(addpath))
 
 	def __len__ (self):
 		return len(self.pack())
 
 	def __str__ (self):
-		return "MP_UNREACH_NLRI Family %s %d NLRI(s)" % (Address.__str__(self.routes[0]),len(self.routes))
+		return "MP_UNREACH_NLRI %d NLRI(s)" % len(self.nlris)
