@@ -6,25 +6,21 @@ Created by Thomas Mangin on 2009-11-05.
 Copyright (c) 2009-2013 Exa Networks. All rights reserved.
 """
 
+from collections import deque
+
 from exabgp.protocol.family import AFI
+
 from exabgp.bgp.message.open.holdtime import HoldTime
 from exabgp.bgp.message.open.capability import AddPath
-from exabgp.bgp.message.update.attribute.id import AttributeID
 
-from exabgp.structure.api import APIOptions
+from exabgp.reactor.api.encoding import APIOptions
 
-from exabgp.rib.watchdog import Watchdog
-
-from exabgp.structure.log import Logger
+from exabgp.rib import RIB
 
 # The definition of a neighbor (from reading the configuration)
 class Neighbor (object):
-	# This need to be global, all neighbor share the same data !
-	# It allows us to use the information when performing global routing table update calculation
-	watchdog = Watchdog()
-
 	def __init__ (self):
-		self.logger = Logger()
+		# self.logger should not be used here as long as we do use deepcopy as it contains a Lock
 		self.description = ''
 		self.router_id = None
 		self.local_address = None
@@ -37,86 +33,68 @@ class Neighbor (object):
 		self.md5 = None
 		self.ttl = None
 		self.group_updates = None
+		self.flush = None
+		self.adjribout = None
 
 		self.api = APIOptions()
 
+		self.passive = False
+
 		# capability
 		self.route_refresh = False
 		self.graceful_restart = False
 		self.multisession = None
 		self.add_path = None
+		self.aigp = None
 
 		self._families = []
-		self._routes = {}
+		self.rib = None
+
+		self.operational = None
+		self.asm = dict()
+
+		self.messages = deque()
+		self.refresh = deque()
+
+	def make_rib (self):
+		self.rib = RIB(self.name(),self.adjribout,self._families)
+
+	# will resend all the routes once we reconnect
+	def reset_rib (self):
+		self.rib.reset()
+		self.messages = deque()
+		self.refresh = deque()
+
+	# back to square one, all the routes are removed
+	def clear_rib (self):
+		self.rib.clear()
+		self.messages = deque()
+		self.refresh = deque()
 
 	def name (self):
 		if self.multisession:
-			session = "/ ".join("%s-%s" % (afi,safi) for (afi,safi) in self.families())
+			session = '/'.join("%s-%s" % (afi.name(),safi.name()) for (afi,safi) in self.families())
 		else:
 			session = 'in-open'
-		return "%s local-ip %s local-as %s peer-as %s router-id %s family-allowed %s" % (self.peer_address,self.local_address,self.local_as,self.peer_as,self.router_id,session)
+		return "neighbor %s local-ip %s local-as %s peer-as %s router-id %s family-allowed %s" % (self.peer_address,self.local_address,self.local_as,self.peer_as,self.router_id,session)
 
 	def families (self):
 		# this list() is important .. as we use the function to modify self._families
 		return list(self._families)
 
-	def every_routes (self):
-		for family in list(self._routes.keys()):
-			for route in self._routes[family]:
-				yield route
-
-	def routes (self):
-		# This function returns a hash and not a list as "in" tests are O(n) with lists and O(1) with hash
-		# and with ten thousands routes this makes an enormous difference (60 seconds to 2)
-
-		def _routes (self):
-			for family in list(self._routes.keys()):
-				for route in self._routes[family]:
-					yield route
-
-		routes = {}
-		for route in self.watchdog.filtered(_routes(self)):
-			routes[route.index()] = route
-		return routes
-
 	def add_family (self,family):
+		# the families MUST be sorted for neighbor indexing name to be predictable for API users
 		if not family in self.families():
-			self._families.append(family)
+			afi,safi = family
+			d = dict()
+			d[afi] = [safi,]
+			for afi,safi in self._families:
+				d.setdefault(afi,[]).append(safi)
+			self._families = [(afi,safi) for afi in sorted(d) for safi in sorted(d[afi])]
 
-	def remove_family_and_routes (self,family):
+	def remove_family (self,family):
 		if family in self.families():
 			self._families.remove(family)
-			if family in self._routes:
-				del self._routes[family]
-
-	def add_route (self,route):
-		self.watchdog.integrate(route)
-		self._routes.setdefault((route.nlri.afi,route.nlri.safi),set()).add(route)
-
-	def remove_route (self,route):
-		try :
-			removed = False
-			routes = self._routes[(route.nlri.afi,route.nlri.safi)]
-			if route.nlri.afi in (AFI.ipv4,AFI.ipv6):
-				for r in list(routes):
-					if r.nlri == route.nlri and \
-					   r.attributes.get(AttributeID.NEXT_HOP,None) == route.attributes.get(AttributeID.NEXT_HOP,None):
-						routes.remove(r)
-						removed = True
-			else:
-				routes.remove(route)
-				removed = True
-		except KeyError:
-			removed = False
-		return removed
-
-	def set_routes (self,routes):
-		# routes can be a generator for self.everyroutes, if we delete self._families
-		# then the generator will return an empty content when ran, so we can not use add_route
-		f = {}
-		for route in routes:
-			f.setdefault((route.nlri.afi,route.nlri.safi),set()).add(route)
-		self._routes = f
 
 	def missing (self):
 		if self.local_address is None: return 'local-address'
@@ -134,6 +112,7 @@ class Neighbor (object):
 			self.local_as == other.local_as and \
 			self.peer_address == other.peer_address and \
 			self.peer_as == other.peer_as and \
+			self.passive == other.passive and \
 			self.hold_time == other.hold_time and \
 			self.md5 == other.md5 and \
 			self.ttl == other.ttl and \
@@ -141,29 +120,33 @@ class Neighbor (object):
 			self.graceful_restart == other.graceful_restart and \
 			self.multisession == other.multisession and \
 			self.add_path == other.add_path and \
+			self.operational == other.operational and \
 			self.group_updates == other.group_updates and \
+			self.flush == other.flush and \
+			self.adjribout == other.adjribout and \
 			self.families() == other.families()
 
 	def __ne__(self, other):
 		return not self.__eq__(other)
 
-	def pprint (self,with_routes=True):
-		routes=''
-		if with_routes:
-			routes += '\nstatic { '
-			for route in self.every_routes():
-				routes += '\n    %s' % route
-			route += '\n}'
+	def pprint (self,with_changes=True):
+		changes=''
+		if with_changes:
+			changes += '\nstatic { '
+			for changes in self.rib.incoming.queued_changes():
+				changes += '\n    %s' % changes.extensive()
+			changes += '\n}'
 
 		families = ''
 		for afi,safi in self.families():
 			families += '\n    %s %s;' % (afi.name(),safi.name())
 
 		_api  = []
-		_api.extend(['    neighbor-changes;\n',] if self.api.neighbor_changes else [])
-		_api.extend(['    receive-packets;\n',]  if self.api.receive_packets else [])
-		_api.extend(['    send-packets;\n',]     if self.api.send_packets else [])
-		_api.extend(['    receive-routes;\n',]   if self.api.receive_routes else [])
+		_api.extend(['    neighbor-changes;\n',]    if self.api.neighbor_changes else [])
+		_api.extend(['    receive-packets;\n',]     if self.api.receive_packets else [])
+		_api.extend(['    send-packets;\n',]        if self.api.send_packets else [])
+		_api.extend(['    receive-routes;\n',]      if self.api.receive_routes else [])
+		_api.extend(['    receive-operational;\n',] if self.api.receive_operational else [])
 		api = ''.join(_api)
 
 		return """\
@@ -172,11 +155,11 @@ neighbor %s {
   router-id %s;
   local-address %s;
   local-as %s;
-  peer-as %s;
+  peer-as %s;%s
   hold-time %s;
-%s%s%s
+%s%s%s%s%s
   capability {
-%s%s%s%s%s  }
+%s%s%s%s%s%s%s  }
   family {%s
   }
   process {
@@ -188,8 +171,11 @@ neighbor %s {
 	self.local_address,
 	self.local_as,
 	self.peer_as,
+	'\n  passive;\n' if self.passive else '',
 	self.hold_time,
 	'  group-updates: %s;\n' % self.group_updates if self.group_updates else '',
+	'  auto-flush: %s;\n' % 'true' if self.flush else 'false',
+	'  adj-rib-out: %s;\n' % 'true' if self.adjribout else 'false',
 	'  md5: %d;\n' % self.ttl if self.ttl else '',
 	'  ttl-security: %d;\n' % self.ttl if self.ttl else '',
 	'    asn4 enable;\n' if self.asn4 else '    asn4 disable;\n',
@@ -197,9 +183,11 @@ neighbor %s {
 	'    graceful-restart %s;\n' % self.graceful_restart if self.graceful_restart else '',
 	'    add-path %s;\n' % AddPath.string[self.add_path] if self.add_path else '',
 	'    multi-session;\n' if self.multisession else '',
+	'    operational;\n' if self.operational else '',
+	'    aigp;\n' if self.aigp else '',
 	families,
 	api,
-	routes
+	changes
 )
 
 	def __str__ (self):
